unit uTrueTypeFunc;

interface

uses
  uTrueTypeStruct;

function TTUShortToInt(const Value: TT_USHORT): Word;
function TTShortToInt(const Value: TT_SHORT): SmallInt;
function TTULongToInt(const Value: TT_ULONG): LongWord;
function TTLongToInt(const Value: TT_LONG): LongInt;
procedure TTIntToUShort(Value: Word; out TTVal: TT_USHORT);
procedure TTIntToShort(Value: SmallInt; out TTVal: TT_SHORT);
procedure TTIntToULong(Value: LongWord; out TTVal: TT_ULONG);
procedure TTIntToLong(Value: LongInt; out TTVal: TT_LONG);
function TTWideToWide(Value: Pointer; Length: integer): WideString; overload;
function TTWideToWide(const Value: WideString; Storage: Pointer): Pointer; overload;

implementation

type
  TBytes = array[0..65535] of byte;

procedure Swap(const Source; out Destination; const Size: integer);
var i, j: integer;
begin
  i := 0;
  j := Pred(Size);
  while j >= 0 do
    begin
    TBytes(Destination)[i] := TBytes(Source)[j];
    Inc(i);
    Dec(j);
    end;
end;

function TTUShortToInt(const Value: TT_USHORT): Word;
begin
  Swap(Value, Result, Sizeof(Value));
end;

function TTShortToInt(const Value: TT_SHORT): SmallInt;
begin
  Swap(Value, Result, Sizeof(Value));
end;

function TTULongToInt(const Value: TT_ULONG): LongWord;
begin
  Swap(Value, Result, Sizeof(Value));
end;

function TTLongToInt(const Value: TT_LONG): LongInt;
begin
  Swap(Value, Result, Sizeof(Value));
end;

procedure TTIntToUShort(Value: Word; out TTVal: TT_USHORT);
begin
  Swap(Value, TTVal, Sizeof(Value));
end;

procedure TTIntToShort(Value: SmallInt; out TTVal: TT_SHORT);
begin
  Swap(Value, TTVal, Sizeof(Value));
end;

procedure TTIntToULong(Value: LongWord; out TTVal: TT_ULONG);
begin
  Swap(Value, TTVal, Sizeof(Value));
end;

procedure TTIntToLong(Value: LongInt; out TTVal: TT_LONG);
begin
  Swap(Value, TTVal, Sizeof(Value));
end;

function TTWideToWideInt(Src, Dest: PTT_USHORT; Length: integer): Pointer;
begin
  while Length > 0 do
    begin
    Dest^[0] := Src^[1];
    Dest^[1] := Src^[0];
    Inc(Src);
    Inc(Dest);
    Dec(Length);
    end;
  Result := Dest;
end;

function TTWideToWide(Value: Pointer; Length: integer): WideString;
begin
  SetLength(Result, Length);
  TTWideToWideInt(Value, @(Result[1]), Length);
end;

function TTWideToWide(const Value: WideString; Storage: Pointer): Pointer;
begin
  Result := TTWideToWideInt(@(Value[1]), Storage, Length(Value));
end;

end.
